==> crow-2.15.0.tar.bz2: crow-2.15.0/crow/controller.h <==
/*
 *  Crow, a GUI designer for GTK+
 *  Copyright (c) 2005-2009 Maxim Udushlivy
 *
 *  Permission is hereby granted, free of charge, to any person 
 *  obtaining a copy of this software and associated documentation 
 *  files (the "Software"), to deal in the Software without 
 *  restriction, including without limitation the rights to use, copy, 
 *  modify, merge, publish, distribute, sublicense, and/or sell copies 
 *  of the Software, and to permit persons to whom the Software is 
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be 
 *  included in all copies or substantial portions of the Software.
 *
 *  The software is provided "as is", without warranty of any kind, 
 *  express or implied, including but not limited to the warranties of 
 *  merchantability, fitness for a particular purpose and 
 *  noninfringement. In no event shall the authors or copyright 
 *  holders be liable for any claim, damages or other liability, 
 *  whether in an action of contract, tort or otherwise, arising from, 
 *  out of or in connection with the software or the use or other 
 *  dealings in the software.
 */

#ifndef CROW_CONTROLLER_H
#define CROW_CONTROLLER_H

#include "crow_impl.h"
#include "model.h"

namespace Crow {

class Context {

public:
	virtual ~Context() { }
	
	virtual void sweep() = 0;
	virtual Gtk::Window * getWindow() = 0;
	virtual Settings * getSettings() = 0;
	virtual int runDialog(Gtk::Window * window, bool asdialog=true) = 0;
	virtual void showMessage(const Glib::ustring & str) = 0;

	template <class T> T getKeyValue(const std::string & key) {
		Settings::const_iterator it= getSettings()->find(key);
		CHECK(it != getSettings()->end());
		try {
			return FromString<T>(it->second);
		} catch(const Glib::Exception & x) {
			FatalError(x.what());
			throw;
		}
	}

	template <class T> void setKeyValue(const std::string & key, const T & val) {
		(*getSettings())[key]= ToString<T>(val);
	}

	bool hasKey(const std::string & key);
	void ensureKey(const std::string & key, const std::string & val);
	
};

enum LoadResult { lrSuccess, lrFailed, lrChanged };

class Controller {

public:
	typedef std::map<PNode, PEntityView> ViewMap;

	Controller();
	~Controller();

	Context * getContext() { return context; }
	void setContext(Context * context) { this->context= context; }

	Model * getModel() { return &model; }

	void clearState();
	void begin(UpdateMode um=umNormal);
	void rollback();
	void commit();

	void undo();
	void redo();
	bool canUndo();
	bool canRedo();

	LoadResult load(const Glib::ustring & data);
	void clear();
	Glib::ustring save(const Nodes & nodes);

	Property * findViewProperty(PNode node);
	NodeTest willSave(PNode node);
	bool canDefault(PNode node);

	PEntityView findView(Any value);
	const ViewMap * getViews() { return &views; }
	PEntityView getView(PNode node);
	template <class T> Glib::RefPtr<T> getView(PNode node) { return Glib::RefPtr<T>::cast_dynamic(getView(node)); }

	void touch(PNode node);
	bool isTouched(PNode node);

	void objectDestroyed(PGlibObject object);

	struct EmitterInfo {
		GlibObjectView * view;
		std::string signature;
		Emitter emitter;
	};
	typedef std::multimap<std::string, EmitterInfo> Emitters;
	Emitters getEmitters();

	bool modelGetScalar(PNode node, Any * result, PNode owner, const std::string & name, StateFlags sf);
	bool modelGetVector(PNode node, Any * result, PNode owner, const std::string & name, TypeId base, StateFlags sf);
	bool modelGetEntity(PNode node, Any * result, PNode owner, const std::string & name, TypeId base, Property * property);

	PNode modelSetScalar(PNode owner, const std::string & name, Any value, Property * property);
	PNode modelSetVector(PNode owner, TypeId type, const std::string & name, Any value);
	PNode modelSetEntity(PNode owner, const std::string & name, Any value, Property * property);

protected:
	PNode makeNodeForValue(PNode old, PNode owner, const std::string & name, Any value, Property * property);
	PNode makeLink(PNode old, PNode owner, const std::string & name, Property * property);
	void ensureAndWrite(PNode node, Any value, Property * property);
	PEntityView ensureView(PNode node, Any value=Any());

	void synchronize();
	Nodes getOrderedModified();
	Nodes update();
	void write(const Nodes & updated);
	void setView(PNode node, PEntityView view);
	void removeView(PNode node);
	static void * destroyNotify(void * data);
	void sweepViews();

	bool isVoidEntity(Any value);
	bool isEntity(TypeId type);
	bool isWidget(PNode node);

private:
	Model model;
	Context * context;
	NodeSet touched;

	ViewMap views;

	std::list<PGlibObject> destroyed;

};

}

#endif

==> crow-2.17.1.tar.bz2: crow-2.17.1/crow/controller.h <==
/*
 *  Crow, a GUI designer for GTK+
 *  Copyright (c) 2005-2010 Maxim Udushlivy
 *
 *  Permission is hereby granted, free of charge, to any person 
 *  obtaining a copy of this software and associated documentation 
 *  files (the "Software"), to deal in the Software without 
 *  restriction, including without limitation the rights to use, copy, 
 *  modify, merge, publish, distribute, sublicense, and/or sell copies 
 *  of the Software, and to permit persons to whom the Software is 
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be 
 *  included in all copies or substantial portions of the Software.
 *
 *  The software is provided "as is", without warranty of any kind, 
 *  express or implied, including but not limited to the warranties of 
 *  merchantability, fitness for a particular purpose and 
 *  noninfringement. In no event shall the authors or copyright 
 *  holders be liable for any claim, damages or other liability, 
 *  whether in an action of contract, tort or otherwise, arising from, 
 *  out of or in connection with the software or the use or other 
 *  dealings in the software.
 */

#ifndef CROW_CONTROLLER_H
#define CROW_CONTROLLER_H

#include "crow_impl.h"
#include "model.h"

namespace Crow {

class Context {

public:
	virtual ~Context() { }
	
	virtual void sweep() = 0;
	virtual Gtk::Window * getWindow() = 0;
	virtual Settings * getSettings() = 0;
	virtual int runDialog(Gtk::Window * window, bool asdialog=true) = 0;
	virtual void showMessage(ustring_cr str) = 0;

	template <class T> T getKeyValue(string_cr key) {
		Settings::const_iterator it= getSettings()->find(key);
		CHECK(it != getSettings()->end());
		try {
			return FromString<T>(it->second);
		} catch(const Glib::Exception & x) {
			FatalError(x.what());
			throw;
		}
	}

	template <class T> void setKeyValue(string_cr key, const T & val) {
		(*getSettings())[key]= ToString<T>(val);
	}

	bool hasKey(string_cr key);
	void ensureKey(string_cr key, string_cr val);
	
};

enum LoadResult { lrSuccess, lrFailed, lrChanged };

class Controller {

public:
	typedef std::map<PNode, PEntityView> ViewMap;

	Controller();
	~Controller();

	Context * getContext() { return context; }
	void setContext(Context * context) { this->context= context; }

	Model * getModel() { return &model; }

	void clearState();
	void begin(UpdateMode um=umNormal);
	void rollback();
	void commit();

	void undo();
	void redo();
	bool canUndo();
	bool canRedo();

	LoadResult load(ustring_cr data);
	void clear();
	Glib::ustring save(const Nodes & nodes);

	Property * findViewProperty(PNode node);
	NodeTest willSave(PNode node);
	bool canDefault(PNode node);

	PEntityView findView(Any value);
	const ViewMap * getViews() { return &views; }
	PEntityView getView(PNode node);
	template <class T> Glib::RefPtr<T> getView(PNode node) { return Glib::RefPtr<T>::cast_dynamic(getView(node)); }

	void touch(PNode node);
	bool isTouched(PNode node);

	void objectDestroyed(PGlibObject object);

	struct EmitterInfo {
		GlibObjectView * view;
		std::string signature;
		Emitter emitter;
	};
	typedef std::multimap<std::string, EmitterInfo> Emitters;
	Emitters getEmitters();

	bool modelGetScalar(PNode node, Any * result, PNode owner, string_cr name, StateFlags sf);
	bool modelGetVector(PNode node, Any * result, PNode owner, string_cr name, TypeId base, StateFlags sf);
	bool modelGetEntity(PNode node, Any * result, PNode owner, string_cr name, TypeId base, Property * property);

	PNode modelSetScalar(PNode owner, string_cr name, Any value, Property * property);
	PNode modelSetVector(PNode owner, TypeId type, string_cr name, Any value);
	PNode modelSetEntity(PNode owner, string_cr name, Any value, Property * property);

protected:
	PNode makeNodeForValue(PNode old, PNode owner, string_cr name, Any value, Property * property);
	PNode makeLink(PNode old, PNode owner, string_cr name, Property * property);
	void ensureAndWrite(PNode node, Any value, Property * property);
	PEntityView ensureView(PNode node, Any value=Any());

	void synchronize();
	Nodes getOrderedModified();
	Nodes update();
	void write(const Nodes & updated);
	void setView(PNode node, PEntityView view);
	void removeView(PNode node);
	static void * destroyNotify(void * data);
	void sweepViews();

	bool isVoidEntity(Any value);
	bool isEntity(TypeId type);
	bool isWidget(PNode node);

private:
	Model model;
	Context * context;
	NodeSet touched;

	ViewMap views;

	std::list<PGlibObject> destroyed;

};

}

#endif
